WITH unified_orgs AS (
    SELECT
        -- Get orgs with MDR Unified Console enabled yesterday.
        distinct(org_key),
        org_name
        FROM "product_metrics"."cbc_orgs_config"
        WHERE array_join(transform(flags_enabled, x -> if(x.feature = 'tempMDRUnifiedConsole' and x.enabled = true, 'true', null)), ',') LIKE '%true%' and CAST(sh_date as date) = current_date
),
mdr_alerts AS (
    -- Get alert in last 7 days from orgs with MDR Unified Console enabled yesterday.
    WITH mdr_alerts_all AS (
       SELECT
         triggered_alert1_alert_id_guid alert_id
       , environment
       , org_id
       , org_key
       , triggered_alert1_threat_score severity
       , device_name
       , CAST(device_id AS bigint) device_id
       , "concat"(device_name, ' (', device_id, ')') device
       , actor_process_filename app_name
       , (triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}') create_timestamp
       , triggered_alert1_alert_creation_timestamp create_date_utc
       , "date"(triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}') create_date
       FROM
         (athena.flink_alert_mdr_prod
       LEFT JOIN product_metrics.cbc_orgs USING (org_key))
       WHERE ((triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}' >= (date_trunc('day', current_timestamp AT TIME ZONE '{time_zone}') - INTERVAL  '7' DAY))
                  AND (triggered_alert1_alert_creation_timestamp AT TIME ZONE '{time_zone}' < date_trunc('day', current_timestamp AT TIME ZONE '{time_zone}')))
    ),
    mdr_alerts_updates AS (
    -- Get Alerts' update
       SELECT
           triggered_alert1_alert_id_guid AS alert_id,
           (CASE WHEN (triggered_alert1_mdr_state_details_classification_alert_classification IN ('TRUE_POSITIVE', 'NOT_ENOUGH_INFO')) THEN 'Likely Threat' WHEN (triggered_alert1_mdr_state_details_classification_alert_classification IN ('FALSE_POSITIVE')) THEN 'Unlikely Threat' ELSE 'Not Reviewed' END) AS determination,
           triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp AT TIME ZONE '{time_zone}' AS mdr_last_timestamp
       FROM (
                select
                    triggered_alert1_mdr_state_details_classification_alert_classification,
                    triggered_alert1_alert_id_guid,
                    triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp,
                    row_number() over (
        partition by triggered_alert1_alert_id_guid
        order by triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp desc
    ) as seqnum
                from athena.flink_eedr_alert_user_update_history_prod
                where (event_date >= "substr"(CAST((current_date - INTERVAL  '7' DAY) AS varchar), 1, 10))
                  and triggered_alert1_mdr_state_details_workflow_state_last_update_timestamp is not null
)
where seqnum = 1)
    , mdr_alerts AS (
       SELECT
         alert_id
       , mdr_alerts_all.*
       , (CASE WHEN ((mdr_last_timestamp IS NOT NULL) AND (mdr_last_timestamp > create_timestamp)) THEN "date_diff"('second', create_timestamp, mdr_last_timestamp) ELSE 0 END) full_ttr
       , COALESCE(determination, 'Not Reviewed') determination
       , 'unified' source
       FROM
         (mdr_alerts_all
       LEFT JOIN mdr_alerts_updates USING (alert_id))
    )
    , legacy_mdr_alerts AS (
       SELECT
         sec_id alert_id
       , env environment
       , ts.org_id
       , org_key
       , t_score severity
       , dev_name device_name
       , dev_id device_id
       , "concat"(dev_name, ' (', CAST(dev_id AS varchar), ')') device
       , app_name
       , ("date_add"('second', (create_time / 1000), CAST('1970-01-01' AS timestamp)) AT TIME ZONE '{time_zone}') create_timestamp
       , ("date_add"('second', (create_time / 1000), CAST('1970-01-01' AS timestamp))) create_date_utc
       , "date"("date_add"('second', (create_time / 1000), CAST('1970-01-01' AS timestamp)) AT TIME ZONE '{time_zone}') create_date
       , (CASE WHEN (("element_at"(audit, 1).timestamp > create_time) AND ("element_at"(audit, "cardinality"(audit)).timestamp > create_time)) THEN (("element_at"(audit, "cardinality"(audit)).timestamp - create_time) / 1000) WHEN ("element_at"(audit, "cardinality"(audit)).timestamp > "element_at"(audit, 1).timestamp) THEN (("element_at"(audit, "cardinality"(audit)).timestamp - "element_at"(audit, 1).timestamp) / 1000) WHEN ("element_at"(audit, 1).timestamp > create_time) THEN (("element_at"(audit, 1).timestamp - create_time) / 1000) ELSE 0 END) full_ttr
       , (CASE WHEN ("element_at"(audit, "cardinality"(audit)).close_code IN ('B', 'NB', 'M', 'IE', 'UB')) THEN 'Likely Threat' WHEN ("element_at"(audit, "cardinality"(audit)).close_code IN ('NR')) THEN 'Not Reviewed' WHEN ("element_at"(audit, "cardinality"(audit)).close_code IN ('NT')) THEN 'Unlikely Threat' WHEN ("element_at"(audit, "cardinality"(audit)).action = 'notified') THEN 'Likely Threat' ELSE 'Unlikely Threat' END) determination
       , 'legacy' source
       FROM
         (mrsrobot_db.threatsight_sec ts
       LEFT JOIN product_metrics.cbc_orgs orgs ON ((ts.env = orgs.environment) AND (ts.org_id = orgs.org_id)))
       WHERE alert_source IS NULL
       AND audit IS NOT NULL
       AND audit <> ARRAY[]
       AND date_add('second', (create_time / 1000), CAST('1970-01-01' AS timestamp)) AT TIME ZONE '{time_zone}' >= date_trunc('day', current_timestamp AT TIME ZONE '{time_zone}') - INTERVAL '7' DAY
       AND date_add('second', (create_time / 1000), CAST('1970-01-01' AS timestamp)) AT TIME ZONE '{time_zone}' < date_trunc('day', current_timestamp AT TIME ZONE '{time_zone}')
    )
    SELECT *
    FROM
      mdr_alerts
    UNION (
       SELECT *
       FROM
         legacy_mdr_alerts
       WHERE (NOT (alert_id IN (SELECT alert_id
    FROM
      mdr_alerts
    WHERE (determination <> 'Not Reviewed') and org_key IN (SELECT org_key FROM unified_orgs)
    )))
)
),
likely_ttr AS (
    -- Get TTR from likely alerts for for each day.
    SELECT
        org_key,
        create_date,
        avg(full_ttr) as likely_avg_ttr,
        COUNT(*) as likely_count
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date
),
mdr_ttr AS (
    -- Get TTR from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        avg(full_ttr) as mdr_avg_ttr,
        COUNT(*) as mdr_count
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date
),
likely_alert_stat AS (
    -- Get alert severity distribution from likely alerts for each day.
    SELECT
        org_key,
        create_date,
        COUNT(CASE WHEN severity = 5 THEN 1 END) AS likely_alert_5,
        COUNT(CASE WHEN severity = 6 THEN 1 END) AS likely_alert_6,
        COUNT(CASE WHEN severity = 7 THEN 1 END) AS likely_alert_7,
        COUNT(CASE WHEN severity = 8 THEN 1 END) AS likely_alert_8,
        COUNT(CASE WHEN severity = 9 THEN 1 END) AS likely_alert_9,
        COUNT(CASE WHEN severity = 10 THEN 1 END) AS likely_alert_10
    FROM mdr_alerts
    WHERE determination = 'Likely Threat'
    GROUP BY org_key, create_date
),
mdr_alert_stat AS (
    -- Get alert severity distribution from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        COUNT(CASE WHEN severity = 5 THEN 1 END) AS mdr_alert_5,
        COUNT(CASE WHEN severity = 6 THEN 1 END) AS mdr_alert_6,
        COUNT(CASE WHEN severity = 7 THEN 1 END) AS mdr_alert_7,
        COUNT(CASE WHEN severity = 8 THEN 1 END) AS mdr_alert_8,
        COUNT(CASE WHEN severity = 9 THEN 1 END) AS mdr_alert_9,
        COUNT(CASE WHEN severity = 10 THEN 1 END) AS mdr_alert_10
    FROM mdr_alerts
    WHERE severity >= 5
    GROUP BY org_key, create_date
),
likely_app_freq AS (
    -- Get rank of application and its name from likely alerts for each day.
    SELECT
        org_key,
        create_date,
        MAX(CASE WHEN app_rank = 1 THEN app_name END) AS likely_app_1,
        MAX(CASE WHEN app_rank = 1 THEN app_count END) AS likely_app_1_cnt,
        MAX(CASE WHEN app_rank = 2 THEN app_name END) AS likely_app_2,
        MAX(CASE WHEN app_rank = 2 THEN app_count END) AS likely_app_2_cnt,
        MAX(CASE WHEN app_rank = 3 THEN app_name END) AS likely_app_3,
        MAX(CASE WHEN app_rank = 3 THEN app_count END) AS likely_app_3_cnt,
        MAX(CASE WHEN app_rank = 4 THEN app_name END) AS likely_app_4,
        MAX(CASE WHEN app_rank = 4 THEN app_count END) AS likely_app_4_cnt,
        MAX(CASE WHEN app_rank = 5 THEN app_name END) AS likely_app_5,
        MAX(CASE WHEN app_rank = 5 THEN app_count END) AS likely_app_5_cnt,
        MAX(CASE WHEN app_rank = 6 THEN app_name END) AS likely_app_6,
        MAX(CASE WHEN app_rank = 6 THEN app_count END) AS likely_app_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            app_name,
            COUNT(*) AS app_count,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS app_rank
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date, app_name
    ) ranked_apps
    WHERE app_rank <= 6
    GROUP BY org_key, create_date
),
mdr_app_freq AS (
    -- Get rank of application and its name from MDR alerts for each day.
    SELECT
        org_key,
        create_date,
        MAX(CASE WHEN app_rank = 1 THEN app_name END) AS mdr_app_1,
        MAX(CASE WHEN app_rank = 1 THEN app_count END) AS mdr_app_1_cnt,
        MAX(CASE WHEN app_rank = 2 THEN app_name END) AS mdr_app_2,
        MAX(CASE WHEN app_rank = 2 THEN app_count END) AS mdr_app_2_cnt,
        MAX(CASE WHEN app_rank = 3 THEN app_name END) AS mdr_app_3,
        MAX(CASE WHEN app_rank = 3 THEN app_count END) AS mdr_app_3_cnt,
        MAX(CASE WHEN app_rank = 4 THEN app_name END) AS mdr_app_4,
        MAX(CASE WHEN app_rank = 4 THEN app_count END) AS mdr_app_4_cnt,
        MAX(CASE WHEN app_rank = 5 THEN app_name END) AS mdr_app_5,
        MAX(CASE WHEN app_rank = 5 THEN app_count END) AS mdr_app_5_cnt,
        MAX(CASE WHEN app_rank = 6 THEN app_name END) AS mdr_app_6,
        MAX(CASE WHEN app_rank = 6 THEN app_count END) AS mdr_app_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            app_name,
            COUNT(*) AS app_count,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS app_rank
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date, app_name
    ) ranked_apps
    WHERE app_rank <= 6
    GROUP BY org_key, create_date
),
likely_dev_freq AS (
        -- Get rank of device and its name from likely alerts for each day.
        SELECT
        org_key,
        create_date,
        MAX(CASE WHEN dev_rank = 1 THEN device_name END) AS likely_dev_1,
        MAX(CASE WHEN dev_rank = 1 THEN device_cnt END) AS likely_dev_1_cnt,
        MAX(CASE WHEN dev_rank = 2 THEN device_name END) AS likely_dev_2,
        MAX(CASE WHEN dev_rank = 2 THEN device_cnt END) AS likely_dev_2_cnt,
        MAX(CASE WHEN dev_rank = 3 THEN device_name END) AS likely_dev_3,
        MAX(CASE WHEN dev_rank = 3 THEN device_cnt END) AS likely_dev_3_cnt,
        MAX(CASE WHEN dev_rank = 4 THEN device_name END) AS likely_dev_4,
        MAX(CASE WHEN dev_rank = 4 THEN device_cnt END) AS likely_dev_4_cnt,
        MAX(CASE WHEN dev_rank = 5 THEN device_name END) AS likely_dev_5,
        MAX(CASE WHEN dev_rank = 5 THEN device_cnt END) AS likely_dev_5_cnt,
        MAX(CASE WHEN dev_rank = 6 THEN device_name END) AS likely_dev_6,
        MAX(CASE WHEN dev_rank = 6 THEN device_cnt END) AS likely_dev_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            device_name,
            COUNT(*) AS device_cnt,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS dev_rank
        FROM mdr_alerts
        WHERE determination = 'Likely Threat'
        GROUP BY org_key, create_date, device_name
    ) ranked_apps
    WHERE dev_rank <= 6
    GROUP BY org_key, create_date
),
mdr_dev_freq AS (
        -- Get rank of device and its name from MDR alerts for each day.
        SELECT
        org_key,
        create_date,
        MAX(CASE WHEN dev_rank = 1 THEN device_name END) AS mdr_dev_1,
        MAX(CASE WHEN dev_rank = 1 THEN device_cnt END) AS mdr_dev_1_cnt,
        MAX(CASE WHEN dev_rank = 2 THEN device_name END) AS mdr_dev_2,
        MAX(CASE WHEN dev_rank = 2 THEN device_cnt END) AS mdr_dev_2_cnt,
        MAX(CASE WHEN dev_rank = 3 THEN device_name END) AS mdr_dev_3,
        MAX(CASE WHEN dev_rank = 3 THEN device_cnt END) AS mdr_dev_3_cnt,
        MAX(CASE WHEN dev_rank = 4 THEN device_name END) AS mdr_dev_4,
        MAX(CASE WHEN dev_rank = 4 THEN device_cnt END) AS mdr_dev_4_cnt,
        MAX(CASE WHEN dev_rank = 5 THEN device_name END) AS mdr_dev_5,
        MAX(CASE WHEN dev_rank = 5 THEN device_cnt END) AS mdr_dev_5_cnt,
        MAX(CASE WHEN dev_rank = 6 THEN device_name END) AS mdr_dev_6,
        MAX(CASE WHEN dev_rank = 6 THEN device_cnt END) AS mdr_dev_6_cnt
    FROM (
        SELECT
            org_key,
            create_date,
            device_name,
            COUNT(*) AS device_cnt,
            ROW_NUMBER() OVER (PARTITION BY org_key, create_date ORDER BY COUNT(*) DESC) AS dev_rank
        FROM mdr_alerts
        WHERE severity >= 5
        GROUP BY org_key, create_date, device_name
    ) ranked_apps
    WHERE dev_rank <= 6
    GROUP BY org_key, create_date
)
-- Group alert statistic data by org_key and create_date
SELECT
    org_key,
    unified_orgs.org_name,
    create_date,
    likely_alert_stat.*,
    mdr_alert_stat.*,
    COALESCE(lt.likely_avg_ttr, 0) as likely_avg_ttr,
    COALESCE(lt.likely_count, 0) as likely_count,
    COALESCE(mt.mdr_avg_ttr, 0) as mdr_avg_ttr,
    COALESCE(mt.mdr_count, 0) as mdr_count,
    likely_app.*,
    mdr_app_freq.*,
    likely_dev.*,
    mdr_dev_freq.*
FROM unified_orgs
LEFT JOIN mdr_alert_stat using (org_key)
LEFT JOIN likely_alert_stat using (org_key, create_date)
LEFT JOIN likely_ttr lt using (org_key, create_date)
LEFT JOIN mdr_ttr mt using (org_key, create_date)
LEFT JOIN likely_app_freq likely_app using (org_key, create_date)
LEFT JOIN mdr_app_freq using (org_key, create_date)
LEFT JOIN likely_dev_freq likely_dev using (org_key, create_date)
LEFT JOIN mdr_dev_freq using (org_key, create_date)
